

//	(C) 2017 R-MIK Sawomir Skrzyski
//	slawomir.skrzynski@ep.com.pl


#define UBRR_RST (((F_CPU) + 8UL * (9600)) / (16UL * (9600)) -1UL)
#define UBRR_BIT (((F_CPU) + 8UL * (115200)) / (16UL * (115200)) -1UL)


//=======================================================================//
void UsartOW_Init();
int UsartOW_Receive();
void UsartOW_Transmit( unsigned char data );
void UsartOW_Flush();


//----------------------------------------------------------------------//
byte onewireInit()
{
  #if ONEWIRE_UART == 0
	#ifdef UDR0
	  #define	OW_UBRRH	UBRR0H
	  #define	OW_UBRRL	UBRR0L
	  #define	OW_UCSRA	UCSR0A
	  #define	OW_UCSRB	UCSR0B
	  #define	OW_UCSRC	UCSR0C
	  #define	OW_UDR		UDR0
	  #define	OW_UDRE		UDRE0
	  #define	OW_UPE		UPE0
	  #define	OW_FE		FE0
	  #define	OW_DOR		DOR0
	  #define	OW_U2X		U2X0
	  #define	OW_RXEN		RXEN0
	  #define	OW_TXEN		TXEN0
	  #define	OW_URSEL	URSEL0
	  #define	OW_UCSZ		UCSZ00
	#elif
	  #ifdef UDR
	    #define	OW_UBRRH	UBRRH
	    #define	OW_UBRRL	UBRRL
	    #define	OW_UCSRA	UCSRA
	    #define	OW_UCSRB	UCSRB
	    #define	OW_UCSRC	UCSRC
	    #define	OW_UDR		UDR
	    #define	OW_UDRE		UDRE
	    #define	OW_UPE		UPE
	    #define	OW_FE		FE
	    #define	OW_DOR		DOR
	    #define	OW_U2X		U2X
	    #define	OW_RXEN		RXEN
	    #define	OW_TXEN		TXEN
		#define	OW_URSEL	URSEL
		#define	OW_UCSZ		UCSZ0
	  #else
	    #error "Procesor nie posiada UART."
	  #endif
	#endif
  #elif ONEWIRE_UART == 1
	#define	OW_UBRRH	UBRRH1H
	#define	OW_UBRRL	UBRRH1L
	#define	OW_UCSRA	UCSR1A
	#define	OW_UCSRB	UCSR1B
	#define	OW_UCSRC	UCSR1C
	#define	OW_UDR		UDR1
	#define	OW_UDRE		UDRE1
	#define	OW_UPE		UPE1
	#define	OW_FE		FE1
	#define	OW_DOR		DOR1
	#define	OW_U2X		U2X1
	#define	OW_RXEN		RXEN1
	#define	OW_TXEN		TXEN1
	#define	OW_URSEL	URSEL1
	#define	OW_UCSZ		UCSZ10
  #elif ONEWIRE_UART == 2
	#define	OW_UBRRH	UBRRH2H
	#define	OW_UBRRL	UBRRH2L
	#define	OW_UCSRA	UCSR2A
	#define	OW_UCSRB	UCSR2B
	#define	OW_UCSRC	UCSR2C
	#define	OW_UDR		UDR2
	#define	OW_UDRE		UDRE2
	#define	OW_UPE		UPE2
	#define	OW_FE		FE2
	#define	OW_DOR		DOR2
	#define	OW_U2X		U2X2
	#define	OW_RXEN		RXEN2
	#define	OW_TXEN		TXEN2
	#define	OW_URSEL	URSEL2
	#define	OW_UCSZ		UCSZ20
  #elif ONEWIRE_UART == 3
	#define	OW_UBRRH	UBRRH3H
	#define	OW_UBRRL	UBRRH3L
	#define	OW_UCSRA	UCSR3A
	#define	OW_UCSRB	UCSR3B
	#define	OW_UCSRC	UCSR3C
	#define	OW_UDR		UDR3
	#define	OW_UDRE		UDRE3
	#define	OW_UPE		UPE3
	#define	OW_FE		FE3
	#define	OW_DOR		DOR3
	#define	OW_U2X		U2X3
	#define	OW_RXEN		RXEN3
	#define	OW_TXEN		TXEN3
	#define	OW_URSEL	URSEL3
	#define	OW_UCSZ		UCSZ30
  #else
    #error "Nieobslugiwany UART."
  #endif
  UsartOW_Init();


	return( onewireTest() );
}


//----------------------------------------------------------------------//
byte onewireTest()
{
int bit, ovt;


	onewireReset();


	onewireStrongPullup( FALSE );		// Wycz silne podciganie
	UsartOW_Flush();
  	UsartOW_Transmit( 0xFF );			// Wylij impuls slotu odczytu

	ovt=OWOVT_BIT;						// Timeout = 80us
	while( (bit=UsartOW_Receive()) == NONE )
		{
		_delay_us(1);
		if ( !(--ovt) ){
			return( FALSE ); }			// Timeout
		}
	if ( (bit & 0xFF) != 0 )
		{
		onewireReset();
		return( TRUE );
		}


	return( FALSE );
}


//----------------------------------------------------------------------//
void UsartOW_Init()
{
byte ursel=0;


	#ifdef __AVR_ATmega162__
	  ursel = (1<<OW_URSEL);
	  OW_UCSRC &= ~(1<<OW_URSEL);
	#endif
	OW_UBRRH = UBRR_BIT >> 8;				// Set baud
	OW_UBRRL = UBRR_BIT & 0xFF;

	OW_UCSRC = (3<<OW_UCSZ) | ursel;		// Set frame format: 8n1
	OW_UCSRB = (1<<OW_RXEN) | (1<<OW_TXEN)	// Enable receiver and transmitter
					/*	| (1<<OW_RXCIE)		// Waczenie przerwa odborczych
						| (1<<OW_TXCIE) */ 	// Waczenie przerwa nadawczych
	;
	#if USE_2X
	 OW_UCSRA |= OW_U2X;
	#endif
}


//----------------------------------------------------------------------//
int UsartOW_Receive()
{
int znak, err=0;


//	TimeOutRs0 = OVER_TIME_RS0;									// TimeOut
//	do
//		{
		if ( (OW_UCSRA & (1<<RXC0)) )								// Jeli odernao znak to odczytaj go i wyjd z funkcji
			{
 			if ( (OW_UCSRA & (1<OW_UPE)) ){ err = PARITYERR; } 		// Sprawdzenie parzystoci
			if ( (OW_UCSRA & (1<OW_FE)) ) { err = FRAMEERR; } 		// Sprawdzenie bdu ramki (np zy znak stopu)
			if ( (OW_UCSRA & (1<OW_DOR)) ){ err = OVERRUNERR; } 	// Sprawdzenie przepenienia bufora
			znak = OW_UDR | err;
			return( znak );
			}
//		}
//	while ( TimeOutRs0 != 0 );

	return ( NONE );			// Min timeout, wyjd z bdem
}


//----------------------------------------------------------------------//
void UsartOW_Transmit( unsigned char data )
{
	/* Wait for empty transmit buffer */
	while ( !( OW_UCSRA & (1<<OW_UDRE)) ){ wdt_reset(); }
	/* Put data into buffer, sends the data */
	OW_UDR = data;
}


//----------------------------------------------------------------------//
void UsartOW_Flush()
{
	while( UsartOW_Receive() != NONE ) ;	// Czyszczenie FIFO
}


//----------------------------------------------------------------------//
/*
 * read one bit
 */
uint8_t onewireReadBit(void){
uint8_t dsBit;
byte ovt;
int bit;


	onewireStrongPullup( FALSE );		// Wycz silne podciganie


	UsartOW_Flush();
  	UsartOW_Transmit( 0xFF );			// Wylij impuls slotu odczytu
	ovt=OWOVT_BIT;						// Timeout = 80us

	while( (bit=UsartOW_Receive()) == NONE )
		{
		_delay_us(1);
		if ( !(--ovt) )
			return( 1 );				// Timeout
		}
	if ( ovt ) while( ovt-- ) _delay_us(1);	// Czas na naadowanie pojemnoci magistrali

	#ifdef OWBTEST
	  #warning "Tablica testowa odczytanych bitw."
	  if ( ow_pb <64 ) tab1wb[ow_pb++] = bit; 	//test
	#endif
	if ( (bit & 0xFF) != 0xFF ) return( 0 );

	return( 1 );
}


//----------------------------------------------------------------------//
/*
 * write one bit
 */
byte onewireWriteBit(uint8_t bit){


	byte ovt;


	UsartOW_Flush();
	// Wylij impuls slotu zapisu
	if ( bit ){	UsartOW_Transmit( 0xFF ); } else { UsartOW_Transmit( 0x00 ); }


	ovt=OWOVT_BIT;							// Timeout = 100us
	while( (bit=UsartOW_Receive()) == NONE )
		{
		_delay_us(1);
		if ( !(--ovt) ){
			return( 0x80 ); }				// Przekroczono Timeout
		}


	return( TRUE );


}


//----------------------------------------------------------------------//
uint8_t onewireReset()
{
uint8_t dsRst;
byte ovt;
int rst;


	#ifdef OWBTEST
	  static ow_pb=0;
	#endif


	#ifdef __AVR_ATmega162__
	  OW_UCSRC &= ~(1<<OW_URSEL);
	#endif
  	OW_UBRRH = UBRR_RST >> 8;
  	OW_UBRRL = UBRR_RST & 0xFF;

	UsartOW_Flush();
  	UsartOW_Transmit( 0xF0 );
	ovt=OWOVT_RST;								// Timeout w 10us
	while( (rst=UsartOW_Receive()) == NONE )
		{
		_delay_us(10);
		if ( --ovt==0 )
			{
			#ifdef __AVR_ATmega162__
			  OW_UCSRC &= ~(1<<OW_URSEL);
			#endif
		  	OW_UBRRH = UBRR_BIT >> 8;
  			OW_UBRRL = UBRR_BIT & 0xFF;
			UsartOW_Flush();
			return ( NONE );					// Timeout
			}
		}
	UsartOW_Flush();
	#ifdef __AVR_ATmega162__
	  OW_UCSRC &= ~(1<<OW_URSEL);
	#endif
  	OW_UBRRH = UBRR_BIT >> 8;
  	OW_UBRRL = UBRR_BIT & 0xFF;

	if ( (rst & 0xFF) == 0xF0 ){
		return( FALSE ); }

	onewireStrongPullup( TRUE );		// Wcz silne podciganie
	_delay_ms(1);
	onewireStrongPullup( FALSE );		// Wycz silne podciganie

	return ( TRUE );
}


//----------------------------------------------------------------------//


