

//******************************************************************//
//																	//
//				(C) 2017 R-MIK Sawomir Skrzyski					//
//				slawomir.skrzynski@ep.com.pl						//
//																	//
//	Programu nie mona rozpowszechnia bez pisemnej zgody autora!	//
//																	//
//******************************************************************//


#include <rmik/wdgi.h>


//======================================================================//
// "czas" - define czau zadziaania WDG, dostpne wartoci:
//			WDTO_30MS, WDTO_60MS, WDTO_120MS, WDTO_250MS, WDTO_500MS, WDTO_1S, WDTO_1S
// "*funkcja - adres funkcji wywoanej przed wyjiem z przerwania od WDG
//			W zmiennej globalnej "adrCallWdg" znajduje sie adres z ktrego nastpio przerwanie WDG.
//			Jeli "funkcja" = 0 to skok nie bdzie wykonany.
// Po zainicjowaniu WDG konieczne jest wczenie przerwa "sei()" w przeciwnym wypadku
// po dwukrotnym przepenieniu timera WDG nastpi restart mikrokontrolera.
//----------------------------------------------------------------------//
void InitWdgI( byte czas, void *funkcja )
{
byte cSREG;


	cSREG = SREG;

	cli();
	InitWdg( czas );
	#if defined(__AVR_ATmega640__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega1281__) || defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__)
	  #ifdef WDGI_NOIRQ
	  	#warning "Zalecane uzycie IRQ od WDG."
	  #endif
	  WDTCSR |= (1<<WDIE);	// Aby ustawi WDIE nie moe by ustawiony bit WDTON w fuses
	  adrUserCallWdg = funkcja;
	  adrUserCallWdg <<= 1;
	#else
	  #ifndef WDGI_NOIRQ
	 	#warning "Tan procesior nie generuje IRQ od WDG. Uzyjto funkcji 'InitWdg()'."
		// #define	WDGI_NOIRQ	// Definicja wyczy obsug IRQ od WDG.
		//  Przydatne dla kompilacji na procesory z i bez tej moliwoci (nie generuje #warnig) gdy procesor nie ma IRQ od WDG.
	  #endif
	#endif

	SREG = cSREG;
}


//----------------------------------------------------------------------//
// "czas" - define czau zadziaania WDG, dostpne wartoci:
//----------------------------------------------------------------------//
void InitWdg( byte czas )
{
byte cSREG;


	cSREG = SREG;
	cli();
	wdt_enable( czas );			// Ustawienie WDG (IRQ musz byc wyczone)
	SREG = cSREG;
}


//----------------------------------------------------------------------//
// WDIE jest automatycznie kasowany dlatego nastpne zadziaanie WDG	//
//   wywoa reset														//
//----------------------------------------------------------------------//
#if defined(__AVR_ATmega164__) || defined(__AVR_ATmega640__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega1281__) || defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__)
#ifdef WDGI_NAKED
  ISR(WDT_vect, ISR_NAKED )
#else
  SIGNAL( WDT_vect )
#endif
{
byte *ptr, ofs=0, adr[3];
word stack, cmd;
#if  defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__)
  long vect;							// Ponad 128K adres jest trzybajtowy
#else
  word vect;
#endif

	#ifndef WDGI_NAKED
	  nop();							// Pierwszy rozkaz procedury (kod maszynowy rozkazu = $0000)
	#endif
	vect = WDT_vect;					// Adres wektora przerwania od WGD
	vect <<= 1;							// Adres w sowach wic mnoymy przez 2

	#ifndef WDGI_NAKED
	  for( byte x=0; x<32; x++ )		// Liczymy liczbe rozkazw push
		{
		#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega1281__) || defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__)
		  cmd = pgm_read_word_far(vect);// Ponad 64KB "far"
		#else
		  cmd = pgm_read_word( vect );
		#endif
		vect +=2;
		if ( !cmd )	break;				// Jeli kod rozkazu "nop"
		if ( ((cmd&0xFF00)==0x9200) ||	// Jeli rozkaz push R0..15
			 ((cmd&0xFF00)==0x9300) )	// Jeli rozkaz push R16..31
			{
			ofs++;
			}
		}
	#endif

	stack = SP + ofs;
	stack++;			// Stos wskazuje pierwszy wolny bajt wic zwikszamy o 1
	ptr = stack;
	adr[0] = *ptr++;
	adr[1] = *ptr++;
	adr[2] = *ptr++;
	#if  defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__)
	  adrCallWdg = ((adr[0] << 8) <<8);	// Adres powrotu dla cpu ponad 128kB
	  adrCallWdg |= adr[1] << 8;
	  adrCallWdg |= adr[2];
	#else
	  adrCallWdg = adr[0] << 8;			// Adres powrotu dla cpu do 128kB
	  adrCallWdg |= adr[1];
	#endif
	adrCallWdg *= 2;					// PC wskazuje nr sowa wic adres jest dwa razy wikszy


	if ( adrUserCallWdg )
		{
		long adrCall = adrUserCallWdg >> 1;	// Rozkaz CALL posuguje si sowami a nie adresem
		(*((void(*)(void))adrCall))();		// CALL do funkcji o adresie "adrUserCallWdg"
		}


#ifdef WDGI_NAKED
  while( true );	// "abort()" nie zadziaa (wycza irq czy wdg - nie pamitam)
#endif
}
#endif


//======================================================================//

